/**
 * JASMINe
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

/**
 * JASMINe
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.jasminemodel.diagram.ui.util;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.Diagnostician;
import org.eclipse.emf.validation.model.EvaluationMode;
import org.eclipse.emf.validation.service.IBatchValidator;
import org.eclipse.emf.validation.service.ModelValidationService;

/**
 * This class allows to validate an Ecore Object. The validation is based on
 * Ecore model integrity and on OCL constraints linked to the model.
 *
 * This validator is not the one called by the 'Validate' action (see
 * {@link org.objectweb.jasmine.ui.jasminemodel.diagram.part.ValidateAction}).
 *
 * @author Julien Legrand
 *
 */
public class JasmineModelValidator {

	// ------------------------------------------------------------------------
	// static methods
	// ------------------------------------------------------------------------

	/**
	 * Validate object and fill troubles in result list
	 *
	 * @param object
	 *            The EObject to validate.
	 * @param result
	 *            A list containing String (formatted like "severity : message")
	 *            describing troubles.
	 * @return The number of properties non-valid.
	 */
	public static int validate(EObject object, List<String> result) {

		if (result == null) {
			result = new ArrayList<String>();
		}

		return validate(object, result, new HashSet<EObject>());

	}

	// ------------------------------------------------------------------------
	// Private methods
	// ------------------------------------------------------------------------

	/**
	 * Validate object and fill troubles in result list
	 *
	 * @param object
	 *            The EObject to validate.
	 * @param result
	 *            A list containing String (formatted like "severity : message")
	 *            describing troubles.
	 * @return The number of properties non-valid.
	 */
	private static int validate(EObject object, List<String> result, Set<EObject> validated) {

		if (result == null) {
			result = new ArrayList<String>();
		}

		/*
		 * check EObject integrity (also check containment objects integrity but
		 * not referenced objects).
		 */
		batchValidator(object, result);
		emfModelValidator(object, result);

		validated.add(object);

		/*
		 * get referenced objects and validate them.
		 */
		EList<EObject> refs = object.eCrossReferences();

		if (!(refs.isEmpty() || refs == null)) {

			for (EObject o : refs) {

				if (! validated.contains(o)) {

					validate(o, result, validated);

				}
			}
		}

		return result.size();

	}

	/**
	 * Validate Ecore object with constraints defined in the GMF mapping audit
	 * container.
	 *
	 * @param object
	 *            Ecore object to validate
	 * @return The validation status
	 */
	private static IStatus batchValidator(EObject object, List<String> result) {

		IBatchValidator validator = (IBatchValidator) ModelValidationService
				.getInstance().newValidator(EvaluationMode.BATCH);
		validator.setIncludeLiveConstraints(true);

		IStatus status = validator.validate(object);
		treatIStatus(status, result);

		return status;
	}

	/**
	 * Validate Ecore object with constraints defined in the EMF model.
	 *
	 * @param object
	 *            Ecore object to validate
	 */
	private static Diagnostic emfModelValidator(EObject object,
			List<String> result) {

		Diagnostic diagnostic = Diagnostician.INSTANCE.validate(object);
		treatDiagnostic(diagnostic, result);

		return diagnostic;
	}

	/**
	 * @param istatus
	 * @param result
	 */
	private static void treatIStatus(IStatus istatus, List<String> result) {

		if (istatus.getSeverity() != 0) {

			StringBuffer sb = new StringBuffer();

			if (istatus.isMultiStatus()) {

				for (IStatus status : istatus.getChildren()) {

					sb.append(severityToString(status.getSeverity()));
					sb.append(" : ");
					sb.append(status.getMessage());

					result.add(sb.toString());

					sb.delete(0, sb.length());
				}
			} else {
				sb.append(severityToString(istatus.getSeverity()));
				sb.append(" : ");
				sb.append(istatus.getMessage());

				result.add(sb.toString());
			}

		}

	}

	/**
	 * @param diagnostic
	 * @param result
	 */
	private static void treatDiagnostic(Diagnostic diagnostic,
			List<String> result) {

		if (diagnostic.getSeverity() != 0) {

			StringBuffer sb = new StringBuffer();

			for (Diagnostic d : diagnostic.getChildren()) {

				sb.append(severityToString(d.getSeverity()));
				sb.append(" : ");
				sb.append(d.getMessage());

				result.add(sb.toString());

				sb.delete(0, sb.length());
			}

		}
	}

	/**
	 * @param severity
	 * @return
	 */
	private static String severityToString(int severity) {

		switch (severity) {
		case 1:
			return "OK";
		case 2:
			return "INFO";
		case 3:
			return "WARNING";
		case 4:
			return "ERROR";
		case 5:
			return "CANCEL";
		default:
			break;
		}
		return null;
	}

}
